/*
 * Created by toText.net.
 * Date: 10/02/2009
 * 
 */
using System;
using System.Net;
using System.Text;
using System.IO;
using System.Threading;

namespace Send
{
	/// <summary>
	/// A class to perform HTTP Posts to toText.net
	/// </summary>
	public class HTTPPoster
	{
		#region Private Member Variables
		/// <summary>The webpage to request as defined the toText.net integration guide</summary>
		private const string SEND_API_STRING = "http://www.totext.net/selfcare/com/zipzaptext/selfcare/SendMessagesApi.action";
		
		/// <summary>API username associated with the account</summary>
		private string	mUsername = "";
		
		/// <summary>API password associated with the account</summary>
		private string	mPassword = "";
		
		/// <summary>The mobile phone number to send the message to (in string form)</summary>
		private string	mDestination = "";
		
		/// <summary>The message to send</summary>
		private string	mMessage = "";
		
		/// <summary>Whether to send the message in test mode</summary>
		private bool?	mTestMode = null;
		
		#endregion Private Member Variables
		
		#region Constructor
		
		/// <summary>
		/// Constructor for the class
		/// </summary>
		/// <param name="username">API username associated with the account</param>
		/// <param name="password">API password associated with the account</param>
		/// <param name="destination">The mobile phone number to send the message to (in string form)</param>
		/// <param name="message">The message to send</param>
		/// <param name="testMode">If true, sends the message in TestMode</param>
		public HTTPPoster(string username, string password, string destination, string message, bool? testMode)
		{
			mUsername = username;
			mPassword = password;
			mDestination = destination;
			mMessage = message;
			mTestMode = testMode;
		}
		
		#endregion Constructor
		
		#region Public Methods
		/// <summary>
		/// Perform the HTTP Post
		/// </summary>
		/// <returns>A response string returned as a result of the post</returns>
		public string Post()
		{
			
			string returnString = "";

			// prepare the web page we will be asking for
			byte[]  buffer = Encoding.ASCII.GetBytes(CreateRequestString());
			HttpWebRequest  request  = CreateAndInitialiseRequest(buffer.Length);
			Stream PostData = request.GetRequestStream();
            //Now we write, and afterwards, we close. Closing is always important!
            PostData.Write(buffer, 0, buffer.Length);
            PostData.Close();
			
			HttpWebResponse response = (HttpWebResponse)request.GetResponse();
			
			// we will read data via the response stream
			Stream resStream = response.GetResponseStream();
			
			returnString = GetStringFromResponseStream(resStream);
	
			response.Close();
			
			return returnString;
			
		}
		
		#endregion Public Methods
		
		#region Private Methods
		
		/// <summary>
		/// Create and init an HTTPWebRequest to make a post with
		/// </summary>
		/// <param name="contentLength">The length of the POST data to follow</param>
		/// <returns>An initialised HTTPWebRequest</returns>
		private HttpWebRequest CreateAndInitialiseRequest(int contentLength)
		{
			// prepare the web page we will be asking for
			HttpWebRequest  request  = (HttpWebRequest)WebRequest.Create(SEND_API_STRING);
			
			request.Credentials = CredentialCache.DefaultCredentials;
			request.Method = "POST";
			request.ContentType ="application/x-www-form-urlencoded";
			request.ContentLength = contentLength;
			return request;
		}
		
		/// <summary>
		/// Extracts a string from a stream
		/// </summary>
		/// <param name="resStream">The stream to extract from</param>
		/// <returns>The string extracted</returns>
		private static string GetStringFromResponseStream(Stream resStream)
		{
			
			string tempString = null;
			int    count      = 0;
			
			// used to extract the string
			StringBuilder sb  = new StringBuilder();
			
			// used on each read operation
			byte[]        buf = new byte[8192];
			
			do
			{
				// fill the buffer with data
				count = resStream.Read(buf, 0, buf.Length);
	
				// make sure we read some data
				if (count != 0)
				{
					// translate from bytes to ASCII text
					tempString = Encoding.ASCII.GetString(buf, 0, count);
	
					// continue building the string
					sb.Append(tempString);
				}
			}
			while (count > 0); // any more data to read?
			
			return sb.ToString();
		}
		
		/// <summary>
		/// Generate a string of data to post from the construction parameters
		/// </summary>
		/// <returns>The string to post</returns>
		private string CreateRequestString()
		{
			string returnString = string.Format("username={0}&password={1}&destination={2}&message={3}",mUsername, mPassword, mDestination,mMessage);
			if(mTestMode.HasValue)
			{
				returnString = string.Format("{0}&testMode={1}",returnString,(bool)mTestMode?1:0);
			}
			return returnString;
			
		}
		
		#endregion Private Methods
	}
}
